#!/usr/bin/env python3
"""Define the :class:`.Spark3D` object."""
import os
import shlex
from pathlib import Path
from subprocess import PIPE, Popen
from typing import Literal

import numpy as np
import pandas as pd
from numpy.typing import NDArray
from spark3dbatch.helper import printc

MODES = Literal[
    "Multipactor",
    "Video Multipactor",
    "Corona",
    "Video Corona",
    "--validate",
    "--list",
    "--config",
]
RF_UNITS_T = Literal["m", "mm", "inches"]
RF_UNITS = ("m", "mm", "inches")

FIELD_MAPS = (".dsp", ".f3e", ".mfe")


class Spark3D:
    """Spark3D simulation object."""

    _installation_folder = Path(os.environ.get("SPARK3DPATH", ""))
    _spark3d_bin = _installation_folder / "./spark3d"

    def __init__(
        self,
        project_path: Path,
        *files: str,
        output_path: Path | None = None,
        **kwargs,
    ) -> None:
        """Constructor.

        Parameters
        ----------
        project_path :
            Folder where the ``SPKX`` is stored.
        args :
            TODO
        output_path :
            Where results are stored. The default is None, which is changed to
            ``project_path`` during object construction.

        """
        if not self._installation_folder.is_dir():
            raise FileNotFoundError(
                "The `SPARK3DPATH` environment variable is unset or points to "
                "a non-existing folder.\n"
                f"$SPARK3DPATH={str(self._installation_folder)}"
            )
        if not self._spark3d_bin.is_file():
            raise FileNotFoundError(
                "The `spark3d` executable was not found in "
                f"{self._installation_folder}"
            )
        assert project_path.exists()
        self.project_path = project_path

        file_name, base_command = self._handle_different_input_types(
            *files, **kwargs
        )
        self.file_name = file_name
        self.input = project_path / file_name
        self.base_command = base_command

        self.output_path = (
            output_path
            if output_path is not None
            else self.project_path / Path(file_name).stem
        )

        self.results_path: Path | None = None
        # created by self._get_results_dir in self.run (path depends on the
        # configuration)

    def run(self, mode: MODES, config: dict[str, int] | None = None) -> None:
        """Launch the project.

        Parameters
        ----------
        mode :
            Type of simulation.
        config :
            Holds the information on the geometry, etc.

        """
        cmd = self._get_cmd(mode, config)
        if config is None:
            config = {}

        printc("Spark3D.run info:", f"running SPARK3D with command\n{cmd}")
        try:
            with Popen(
                cmd,
                shell=True,
                env=os.environ,
                stdout=PIPE,
                stderr=PIPE,
                universal_newlines=True,
            ) as proc:
                assert proc.stdout is not None
                for line in proc.stdout:
                    print(line, end="")
            printc(
                "Spark3D.run info:",
                f"run finished with return code {proc.returncode}.",
            )

            # FIXME
            if proc.returncode != 0:
                printc(
                    "Spark3D.run warning:",
                    "this return code means that an error ocurred during "
                    "SPARK3D execution. Try to manually copy-paste the SPARK3D"
                    " command to output more information.",
                )

        except OSError as err:
            printc("Spark3D.run error:", str(err))

    def get_full_results(
        self,
    ) -> tuple[pd.DataFrame | None, pd.DataFrame | None]:
        """Get the entire simulation results.

        .. todo::
            Auto-detection of results folder.

        Returns
        -------
        pd_power :
            Holds breakdown power, status (BD or noBD) and 'Multipactor order'.
        pd_time :
            Holds simulation number, power, and number of particles vs. time
            for every simulation (every power).

        """
        # TODO auto detect where the results are stored
        assert self.results_path is not None
        # old versions of SPARK3D
        # res_dir = self.results_path / "region1" / "signal1"
        res_dir = self.results_path / "region1" / "signalCW 1"
        assert res_dir.exists(), f"{res_dir} does not exist."

        res: list[pd.DataFrame | None]
        res = [None, None]
        for i, file in enumerate(["power_results.txt", "time_results.txt"]):
            file = res_dir / file
            if file.is_file():
                res[i] = pd.read_csv(file, delimiter="\t", na_values="---")

        pd_power, pd_time = res
        return pd_power, pd_time

    def get_results(self) -> tuple[NDArray, NDArray]:
        """Get a resume of results.

        Returns
        -------
        freq :
            Array of frequencies in Hz.
        power :
            Array of corresponding breakdown powers in W.

        """
        freq, power = None, None
        assert self.results_path is not None
        file = self.results_path / "general_results.txt"
        if file.is_file():
            freq, power = np.loadtxt(
                file, skiprows=1, delimiter="\t", usecols=(3, 4), unpack=True
            )
        assert isinstance(freq, np.ndarray)
        assert isinstance(power, np.ndarray)
        return freq, power

    def _handle_different_input_types(
        self,
        *args: str,
        new_project_name: Path | str | None = None,
        unitsRF: RF_UNITS_T | None = None,
    ) -> tuple[Path, str]:
        """Handle the two types of input in a way that is transparent for user.

        .. todo::
            Check hopw HFSS files actually work

        Parameters
        ----------
        *args :
            Relative path to a ``SPKX`` OR relative path to a ``XML`` file and
            relative path to field map file (``DSP``, ``F3E`` or ``MFE``).
        new_project_name :
            Name of the project constructed from ``XML`` and field map file.
            ``SPKX`` extension must be provided. The default is None (then
            changed to 'my_project.spkx')
        unitsRF :
            Units of the file field map comes from HFSS (``DSP`` I think?).

        Raises
        ------
        IOError
            When the number of input file(s) and/or their type(s) are
            inconsistent.

        Returns
        -------
            The project name, the portion of command line that allows SPARK to
            identify or construct the project.

        """
        paths = [self.project_path / fp for fp in args]
        for path in paths:
            assert path.is_file(), f"Input file {path} does not exist."

        filetypes = [path.suffix for path in paths]

        if len(paths) == 1 and filetypes == [".spkx"]:
            cmd_input = f"--input={paths[0]}"
            return paths[0], cmd_input

        inter = [ext for ext in filetypes if ext in FIELD_MAPS]

        if new_project_name is None:
            new_project_name = "my_project.spkx"

        if len(paths) == 2 and ".xml" in filetypes and len(inter) == 1:
            i_field, i_xml = filetypes.index(inter[0]), filetypes.index(".xml")
            fp_field, fp_xml = paths[i_field], paths[i_xml]

            fp_project = self.project_path / new_project_name
            cmd_input = f"--XMLfile={fp_xml} --importRF={fp_field} "

            if inter[0] == ".dsp":
                assert unitsRF in RF_UNITS, (
                    "The DSP is a HFSS field map file, right? In this case "
                    "you must provide a valid 'unitsRF' key."
                )
                cmd_input += f"unitsRF={unitsRF} "

            cmd_input += f"--projectName={fp_project}"

            return fp_project, cmd_input

        raise OSError(f"Inconsistent input files {args}")

    def _get_cmd(
        self, mode: MODES, config: dict[str, int] | None = None
    ) -> str:
        """Create the command for the project.

        Parameters
        ----------
        mode :
            Type of simulation.
        config :
             Holds the information on the geometry, etc.

        Returns
        -------
        str
            Command to launch SPARK3D.

        """
        if config is None:
            config = {}

        # cmd = [self.BIN_PATH, f"--input={self.input}"]
        cmd = [self._spark3d_bin, self.base_command]

        spkx_kwargs = {
            "--output": self.output_path,
        }

        # No argument required, just validate the integrity of the file or list
        # the valid configurations
        if mode in ("--validate", "--list"):
            cmd.append(mode)
            return " ".join(shlex.quote(str(part)) for part in cmd)

        if mode == "--config":
            my_configuration = self._config_argument(mode, **config)
            self.results_path = self.output_path / self._get_results_dir(
                mode, **config
            )
            cmd.append(f"{mode}={my_configuration}")

            for key, value in spkx_kwargs.items():
                cmd.append(key + "=" + str(value))
            return " ".join(shlex.quote(str(part)) for part in cmd)

        raise OSError(f"configuration {mode} was not recognized.")

    def _config_argument(
        self,
        mode: MODES,
        project: int = 1,
        model: int = 1,
        confs: int = 1,
        em_conf: int = 1,
        discharge_conf: int = 1,
        video: int = 1,
    ) -> str:
        """Create the argument that goes after ``--config=``.

        Parameters
        ----------
        mode :
            Type of simulation to be performed.
        project :
            Project ID.
        model :
            Model ID.
        confs :
            Configurations ID.
        em_conf :
            EMConfigGroup ID.
        discharge_conf :
            MultipactorConfig or CoronaConfig ID.
        video :
            VideoMultipactorConfig or VideoCoronaConfig.

        Returns
        -------
        str
            Argument that goes after ``--config=``.

        """
        out = [
            f"Project:{project}",
            f"/Model:{model}",
            f"/Configurations:{confs}",
            f"/EMConfigGroup:{em_conf}",
        ]

        d_mode = {
            "Multipactor": f"/MultipactorConfig:{discharge_conf}//",
            "Video Multipactor": f"/MultipactorConfig:{discharge_conf}"
            + f"/VideoMultipactorConfig:{video}//",
            "Corona": f"/CoronaConfig:{discharge_conf}//",
            "Video Corona": f"/CoronaConfig:{discharge_conf}"
            + f"/VideoCoronaConfig:{video}//",
        }
        if mode not in d_mode:
            raise OSError("Invalid mode.")
        out.append(d_mode[mode])
        return "".join(out)

    # TODO: check dirs for Corona and Videos
    def _get_results_dir(
        self,
        mode: MODES,
        project: int = 1,
        model: int = 1,
        confs: int = 1,
        em_conf: int = 1,
        discharge_conf: int = 1,
        video: int = 1,
    ) -> Path:
        """Get the full path to the results folder.

        Parameters
        ----------
        mode :
            Type of simulation to be performed.
        project :
            Project ID.
        model :
            Model ID.
        confs :
            Configurations ID.
        em_conf :
            EMConfigGroup ID.
        discharge_conf :
            MultipactorConfig or CoronaConfig ID.
        video :
            VideoMultipactorConfig or VideoCoronaConfig.

        Returns
        -------
        Path
            Path to the results.

        """
        out = [
            "Results",
            f"@Mod{model}",
            f"@ConfGr{confs}",
            f"@EMConfGr{em_conf}",
        ]

        d_mode = {
            "Multipactor": [f"@MuConf{discharge_conf}"],
            "Video Multipactor": [
                f"@MuConf{discharge_conf}",
                f"@Video{video}",
            ],
            "Corona": [f"@CoConf{discharge_conf}"],
            "Video Corona": [f"@CoConf{discharge_conf}", f"@Video{video}"],
        }
        out.extend(d_mode[mode])
        path = os.path.join(*out)
        return Path(path)


if __name__ == "__main__":
    import tempfile
    from importlib import resources

    output_path = Path(tempfile.mkdtemp())

    files = ("Coax_filter_CST(M, C, Eigenmode).spkx",)
    new_project_name = None

    # files = ("Coax_filter_CST(M, C, Eigenmode).xml", "Coax_filter_CST(M, C, Eigenmode).f3e")
    # new_project_name = output_path / "new_project.spkx"

    with resources.as_file(
        resources.files("spark3dbatch.data")
    ) as project_path:
        spk = Spark3D(
            project_path,
            *files,
            output_path=output_path,
            new_project_name=new_project_name,
        )

        mode = "--validate"
        config = {
            "project": 1,
            "model": 1,
            "confs": 1,
            "em_conf": 1,
            "discharge_conf": 1,
            "video": -1,
        }

        spk.run(mode, config)
        if mode == "--config":
            my_power, my_time = spk.get_full_results()



#!/usr/bin/env python3
"""Define the class holding a SPARK3D configuration.

.. todo::
    Easily allow corona, videos.

"""
import xml.etree.ElementTree as ET
from collections.abc import Sequence
from pathlib import Path

import numpy as np
from spark3dbatch.helper import fmt_array, printc


class SparkXML:
    """A class to handle the ``XML`` files from SPARK3D."""

    #: Links ``d_conf`` keys with :class:`.ElementTree` names
    _convert = {
        "Project": "project",
        "Model": "model",
        "Configurations": "confs",
        "EMConfigGroup": "em_conf",
        "MultipactorConfig": "discharge_conf",
    }

    def __init__(self, file: Path, keys: Sequence[str] | None = None) -> None:
        """Init object.

        Parameters
        ----------
        file :
            Path to the ``XML`` file.
        keys :
            Sequence of strings to indicate where the configuration should be
            in the ``XML`` file.

        """
        self.file = file
        self._tree = ET.parse(file)
        self._spark = self._tree.getroot()

        # Add a VideoMultipactorConfig key if needed, or change Multipactor to
        # corona
        self._keys = (
            tuple(keys)
            if keys is not None
            else (
                "Project",
                "Model",
                "Configurations",
                "EMConfigGroup",
                "MultipactorConfig",
            )
        )

    def get_config(self, **kwargs: int) -> ET.Element:
        """Return the configuration corresponding to the inputs.

        Parameters
        ----------
        *kwargs :
            Links configuration entries from ``self._keys`` to their value.

        Raises
        ------
        IOError
            ``*args`` matched no existing configuration. If it matched several,
            either the ``XML`` is wrong, either this code is wrong!

        Returns
        -------
        ET.Element
            Configuration in the form of an :class:`.ElementTree.Element`.

        """
        keys_xml = (self._convert[key] for key in self._keys)
        values_xml = (kwargs[key] for key in keys_xml)

        path = (f"{key}[{val}]" for key, val in zip(self._keys, values_xml))
        configuration = self._spark.findall("/".join(path))
        if len(configuration) != 1:
            raise OSError("More than one or no configuration was found.")
        return configuration[0]

    def edit(
        self,
        conf: ET.Element,
        save: bool = False,
        info: bool = True,
        **kwargs,
    ) -> None:
        """Modify the ``XML``.

        Parameters
        ----------
        conf :
            Configuration to be modified.
        save :
            To save the updated ``XML`` file. Previous file will be
            overwritten.
        info :
            To output some information on what was changed.
        **kwargs :
            Dict of values to change. Keys must be in ``<MultipactorConfig>``,
            eg ``'initialNumberElectrons'``. To modify inner keys, you must use
            the full path, eg ``'sweepPoints'`` will not work but
            ``'PowerSweep/sweepPoints'`` will. You must ensure that the type of
            the values matches what SPARK3D expects.

        """
        if info:
            printc(
                "xml_parser.edit info:",
                f"Modifying {conf.find('name').text}...",
            )
        for key, new_value in kwargs.items():
            s_old_value = conf.find(key).text

            s_new_val = str(new_value)
            conf.find(key).text = s_new_val
            if info:
                printc(
                    "xml_parser.edit info:",
                    f"Changed {key}: {s_old_value} to {s_new_val}",
                )

        if save:
            self._tree.write(self.file)
            printc("xml_parser.edit info:", f"xml saved in {self.file}")


if __name__ == "__main__":
    from importlib import resources

    file = (
        resources.files("spark3dbatch.data")
        / "Coax_filter_CST(M, C, Eigenmode).xml"
    )
    xml = SparkXML(file)

    # As already defined
    config = {
        "project": 1,
        "model": 1,
        "confs": 1,
        "em_conf": 1,
        "discharge_conf": 1,
        "video": -1,
    }
    xml_conf = xml.get_config(**config)

    power = fmt_array(np.linspace(1e-2, 1e2, 10))

    alter_conf = {
        "initialNumberElectrons": int(2e4),
        "pathRelativePrecision": 0.1,
        "PowerSweep/sweepPoints": power,
    }
    # Warning, save=True will overwrite previous ``XML``.
    xml.edit(xml_conf, save=False, **alter_conf)



"""Provide helper functions."""

import math
from typing import Literal

from numpy.typing import NDArray

COLORS = Literal["red", "blue", "green", "magenta", "cyan", "normal"]


def e_acc_to_power(
    e_acc: NDArray, freq: float, r_q: float, l_acc: float
) -> NDArray:
    r"""Convert array of accelerating fields to RMS powers.

    .. important::
       SPARK3D uses peak power, but their definition is non-conventional.
       With single carriers, it corresponds to the classic RMS power, or half
       the classic peak power.

    Parameters
    ----------
    e_acc :
        Array of accelerating fields in :unit:`V/m`.
    freq :
        Frequency in :unit:`Hz`.
    r_q :
        Shunt impedance over quality factor ratio in :unit:`\\Omega`.
    l_acc :
        Accelerating length in :unit:`m`.

    Returns
    -------
        Array of powers in :unit:`W`.

    """
    omega_0 = 2.0 * math.pi * freq
    peak_power = (l_acc * e_acc) ** 2 / (omega_0 * r_q)
    return 0.5 * peak_power


def fmt_array(p_s: NDArray) -> str:
    """Convert numpy array to a string that SPARK3D can understand.

    Parameters
    ----------
    p_s :
        Array of powers.

    Returns
    -------
    str
        List of floats as understood by SPARK3D.

    """
    return ";".join(f"{x:.12g}" for x in p_s)


def printc(*args: str, color: COLORS = "cyan"):
    """Print colored messages."""
    dict_c = {
        "red": "\x1b[31m",
        "blue": "\x1b[34m",
        "green": "\x1b[32m",
        "magenta": "\x1b[35m",
        "cyan": "\x1b[36m",
        "normal": "\x1b[0m",
    }
    print(dict_c[color] + args[0] + dict_c["normal"], end=" ")
    for arg in args[1:]:
        print(arg, end=" ")
    print("")



#!/usr/bin/env python3
"""Prepare and create a release.

Usage:

.. code-block:: bash

   python release.py <X.Y.Z>

"""
import re
import subprocess
import sys
from collections.abc import Sequence
from datetime import date
from pathlib import Path

import yaml


def run(
    cmd: Sequence[str], check: bool = True, text: bool = True, **kwargs
) -> subprocess.CompletedProcess:
    """Run a shell command and ensure it completes successfully.

    Parameters
    ----------
    cmd :
        The command and its arguments to run.

    Returns
    -------
    subprocess.CompletedProcess
        The result of the completed subprocess.

    """
    return subprocess.run(cmd, check=check, text=text, **kwargs)


def git_clean() -> bool:
    """Check if the Git working directory is clean.

    Returns
    -------
    bool
        True if the working directory is clean, False otherwise.

    """
    result = run(["git", "status", "--porcelain"], capture_output=True)
    return result.stdout.strip() == ""


def current_branch() -> str:
    """Give branch we are on."""
    return run(
        ["git", "rev-parse", "--abbrev-ref", "HEAD"], capture_output=True
    ).stdout.strip()


def on_appropriate_branch(version: str) -> bool:
    """Verify that the version number is consistent with current branch.

    If the version is ``1.2.3``, we should be on the ``1.2.x`` branch.

    Parameters
    ----------
    version :
        The version string to check, in the form ``X.Y.Z``.

    Returns
    -------
    bool
        True if the current Git branch matches ``X.Y.x``, False otherwise.

    """
    try:
        branch = current_branch()
    except subprocess.CalledProcessError:
        print("Failed to get current Git branch.")
        return False

    major, minor, _ = version.split(".")
    expected_branch = f"{major}.{minor}.x"

    if branch != expected_branch:
        print(
            f"You are on branch '{branch}', but version {version} "
            f"suggests you should be on '{expected_branch}'."
        )
        return False

    print(f"Branch '{branch}' matches version {version}.")
    return True


def has_staged_changes() -> bool:
    """Tell if there are changes to commit to avoid errors."""
    result = run(["git", "diff", "--cached", "--quiet"], check=False)
    return result.returncode != 0


def update_files(version: str) -> None:
    """Update :file:`CITATION.cff` and :file:`CHANGELOG.md`.

    Parameters
    ----------
    version :
        The version string to set in the files.

    """
    today = date.today().isoformat()
    # _update_citation(version, today)
    _update_changelog(version, today)


def _update_citation(version: str, today: str) -> None:
    """Update the :file:`CITATION.cff` with the given version and current date.

    Parameters
    ----------
    version :
        The version string to set in the CITATION.cff file.
    today :
        Today's date.

    """
    path = Path("CITATION.cff")
    if not path.exists():
        print("CITATION.cff not found.")
        sys.exit(1)
    with path.open() as f:
        data = yaml.safe_load(f)

    data["version"] = version
    data["date-released"] = today

    with path.open("w") as f:
        yaml.dump(data, f, sort_keys=False, allow_unicode=True)

    run(["git", "add", str(path)])


def _extract_changelog_section(
    version: str, content: str
) -> tuple[str, int, re.Match[str]] | None:
    """Find the changelog section for a version.

    Parameters
    ----------
    version :
        The version string (e.g., ``"1.8.0"``).
    content :
        The full text content of :file:`CHANGELOG.md`.

    Returns
    -------
    tuple[str, int, re.Match[str]] | None
        A tuple of (matched section string, line index, match object) or None
        if not found.

    """
    pattern = rf"^## \[{re.escape(
        version
    )}\](?: - (unreleased|\d{{4}}-\d{{2}}-\d{{2}}))?$"
    lines = content.splitlines()
    for i, line in enumerate(lines):
        if match := re.match(pattern, line, re.IGNORECASE):
            section_lines = [line]
            for j in range(i + 1, len(lines)):
                if lines[j].startswith("## "):
                    break
                section_lines.append(lines[j])
            return ("\n".join(section_lines), i, match)
    return None


def changelog_is_ok(version: str) -> bool:
    """Print the section of the changelog corresponding to the given version.

    Parameters
    ----------
    version :
        The version string whose section should be printed.

    Returns
    -------
    bool :
        If the following processes can continue.

    """
    path = Path("CHANGELOG.md")
    if not path.exists():
        print("CHANGELOG.md not found.")
        return False

    content = path.read_text()
    result = _extract_changelog_section(version, content)
    if not result:
        print(
            f"CHANGELOG.md does not contain '## [{version}]'. You may have "
            "forgotten to update it."
        )
        return False

    section, _, _ = result
    print(f"\nThe CHANGELOG section corresponding to {version} reads:")
    print(
        "   (note that the current date will be automatically appended to the "
        "version\n   number line if it is not present)"
    )
    print("=" * 79)
    print(section)
    print("=" * 79 + "\n")
    return True


def _update_changelog(version: str, today: str) -> None:
    """Add current date to version section name in :file:`CHANGELOG.md`.

    Date is appended to the line containing `## [X.Y.Z]`. If the date is
    already present, check that it matches ``today``.

    Parameters
    ----------
    version :
        The version string to set in the :file:`CHANGELOG.md` file.
    today :
        Today's date.

    """
    path = Path("CHANGELOG.md")
    if not path.exists():
        print("CHANGELOG.md not found.")
        sys.exit(1)

    content = path.read_text()
    result = _extract_changelog_section(version, content)
    if not result:
        print(f"No section '## [{version}]' found in CHANGELOG.md.")
        sys.exit(1)

    _, index, match = result
    matched_date_or_flag = match.group(1)
    if matched_date_or_flag is None:
        print(f"No date yet, appending today's date to version {version}.")
    elif matched_date_or_flag.lower() == "unreleased":
        print(f"'unreleased' found, replacing it with today's date.")
    elif matched_date_or_flag != today:
        print(
            f"Version {version} already has date {matched_date_or_flag}, but "
            f"today is {today}. I will write today's date in CHANGELOG."
        )
        ask_user_to_continue()
    else:
        print(
            "CHANGELOG.md already contains correct date for version "
            f"{version}."
        )
        return

    lines = content.splitlines()
    lines[index] = f"## [{version}] - {today}"

    path.write_text("\n".join(lines) + "\n")
    run(["git", "add", str(path)])


def ask_user_to_continue(
    question: str = "Is it ok for you? (y/n) ",
    error_msg: str = "Operation aborted by user.",
    count: int = 0,
) -> None:
    """Ask the user if he/she wants to continue execution of the script."""
    if count >= 3:
        print("Too many unsuccessful attempts.")
        sys.exit(1)
    answer = input(question)
    if answer.lower() == "y":
        return
    if answer.lower() == "n":
        print(error_msg)
        sys.exit(1)

    print(f"{answer = } was not understood. Please try again.")
    return ask_user_to_continue(count=count + 1)


def main() -> None:
    """Run the release process.

    This includes:
    #. Checking the Git status
    #. Checking the corresponding section in :file:`CHANGELOG.md`
    #. Updating :file:`CITATION.cff` and :file:`CHANGELOG.md`
    #. Committing changes, tagging the release, pushing to origin
    #. Switching to main, merging, pushing

    """
    if len(sys.argv) != 2:
        print("Usage: python release.py X.Y.Z")
        sys.exit(1)

    version = sys.argv[1]
    if not re.fullmatch(r"\d+\.\d+\.\d+", version):
        print("Version must be in format X.Y.Z (e.g. 1.8.0)")
        sys.exit(1)

    tag = f"v{version}"

    if not git_clean():
        print(
            "Git working directory is not clean. Maybe there are uncommited "
            "changes."
        )
        ask_user_to_continue()

    if not on_appropriate_branch(version):
        ask_user_to_continue()

    if not changelog_is_ok(version):
        print("Operation aborted due to a CHANGELOG.md error.")
        sys.exit(1)
    ask_user_to_continue()
    update_files(version)

    if has_staged_changes():
        run(["git", "commit", "-m", f"Prepare release {tag}"])
    else:
        print("Nothing to commit.")

    run(["git", "tag", tag])
    run(["git", "push", "--set-upstream", "origin", current_branch()])
    run(["git", "push", "origin", tag])

    print(f"Release {tag} tagged and pushed!")

    print("I will merge release branch into main and push.")
    ask_user_to_continue()
    run(["git", "checkout", "main"])
    run(["git", "merge", "--no-ff", tag])
    run(["git", "push", "origin", "main"])


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""Provide an example main script.

.. todo::
    Fix paths with importlib

"""
from pathlib import Path

import numpy as np
import spark3dbatch.spark3d as sp
from spark3dbatch.helper import fmt_array
from spark3dbatch.xml_parser import SparkXML

# First simple option: provide a ``SPKX`` file
project = Path("/home/placais/Documents/simulation/python/Spark3DBatch/data/")
args = ("Coax_filter_CST(M, C, Eigenmode).spkx",)
kwargs = {}

# Second option, more complex: provide a ``XML`` and a field map.
#   ``XML`` can be generated by 'unzip project.spkx' and modified afterwards
#   field map can be .f3e, .dsp or .mfe
# project = "/home/placais/Documents/Simulation/work_spark3d/swell"
# args = ("my_Project.xml", "field.f3e")
# kwargs = {"new_project_name": "my_very_new_project.spkx"}

if __name__ == "__main__":
    spk = sp.Spark3D(project, *args, **kwargs)

    # Simulation parameters
    # mode = "--list"
    # mode = "--validate"
    mode = "--config"
    config = {
        "project": 1,
        "model": 1,
        "confs": 1,
        "em_conf": 1,
        "discharge_conf": 1,
        "video": -1,
    }

    selection = 0

    # =========================================================================
    # Simple run
    # =========================================================================
    if selection == 0:
        spk.run(mode, config)
        if mode == "--config":
            my_power, my_time = spk.get_full_results()

    # =========================================================================
    # Same run over and over for convergence
    # =========================================================================
    elif selection == 1:
        counter = 0
        while counter < 10:
            spk.run("--config", config)
            my_freq, my_power = spk.get_results()

            with open("batch_results.txt", "a+", encoding="utf-8") as f_handle:
                np.savetxt(f_handle, [my_power])

            counter += 1

    # =========================================================================
    # Other studies, manually modify the xml
    # =========================================================================
    elif selection == 2:
        assert (
            ".xml" in args[0]
        ), "The edition of the simulation properties requires a ``XML`` + "
        "field map input."

        xml = SparkXML(project / args[0])
        xml_conf = xml.get_config(**config)

        new_power = fmt_array(np.linspace(1e-2, 1e2, 5))
        alter_conf = {
            "initialNumberElectrons": 100,
            "pathRelativePrecision": 0.1,
            "PowerSweep/sweepPoints": new_power,
        }
        # Warning, save=True will overwrite previous ``XML``.
        xml.edit(xml_conf, save=True, **alter_conf)
        spk.run("--validate", config)

        counter = 0
        while counter < 4:
            xml.edit(xml_conf, save=True, **alter_conf)
            spk.run("--config")
            my_freq, my_power = spk.get_results()

            with open("sweep_results.txt", "a+", encoding="utf-8") as f_handle:
                np.savetxt(f_handle, [my_power])

            counter += 1
            alter_conf["initialNumberElectrons"] += 10
